\documentclass{article}
\usepackage{setspace}
\setlength{\textwidth}{6.5in}
\topmargin -1.5cm
\oddsidemargin 0.04cm
\evensidemargin 0.04cm
\textwidth 16.59cm
\textheight 21.94cm 
\begin{document}
\input lambdaTeX
\functions (array,range,bounds,scalar,vector,matrix,result_bounds,transpose,mmap,ones,zeros,matrix_on_scalar,scalar_on_matrix,matrix_on_matrix,scalar_by_matrix,matrix_by_matrix,mbin,mmul,sigmoid,predict)
\begin{fontsize}{5}{5}
\parindent 0pt
\parskip 11pt

\title{Simple Matrix Library:}
\author{(C)2005 - Keean Schupke}
\maketitle

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.\\

\end{fontsize}
\begin{fontsize}{7}{7}

>{-# OPTIONS -fglasgow-exts #-}
>{-# OPTIONS -fallow-undecidable-instances #-}
>
>module Matrix where
>
>import Data.Array.Unboxed
>
>newtype Matrix a e = Matrix (a (Int,Int) e) deriving (Show,Eq)
>type UMatrix e = Matrix UArray e
>
>scalar :: IArray a e => e -> Matrix a e
>scalar a = Matrix $ array ((0,0),(0,0)) [((0,0),a)]
>
>vector :: IArray a e => [e] -> Matrix a e
>vector a = Matrix $ array ((0,0),(0,length a-1)) [((0,i),a!!i) | i <- range (0,length a-1)]
>
>transpose :: (IArray a e, Num e) => Matrix a e -> Matrix a e
>transpose (Matrix a) = Matrix $ array result_bounds [((j,i),a!(i,j)) | i <- range (li,ui), j <- range (lj,uj)] where
>
>	((li,lj),(ui,uj)) = bounds a
>	result_bounds = ((lj,li),(uj,ui))
>
>gen_matrix :: (IArray a e, Num e) => (Int -> Int -> e) -> Int -> Int -> Matrix a e
>gen_matrix f u v = Matrix $ array ((0,0),(u-1,v-1)) [((i,j),f i j) | j <- range (0,v-1), i <- range (0,u-1)]
>
>matrix :: (IArray a e, Num e) => [[e]] -> Matrix a e
>matrix m = Matrix $ array result_bounds [((j,i),((m!!j)!!i)) | i <- range (0,u-1), j <- range (0,v-1)] where
>
>	v = length m
>	u = length (head m)
>	result_bounds
>		| foldl (\b w -> if length w == u then True else False) True m = ((0,0),(v-1,u-1))
>		| otherwise = error "marix: inconsistant row lengths"
>
>ones :: (IArray a e, Num e) => Int -> Int -> Matrix a e
>ones = gen_matrix (\_ _ -> 1)
>
>zeros :: (IArray a e, Num e) => Int -> Int -> Matrix a e
>zeros = gen_matrix (\_ _ -> 0)
>
>mmap :: (IArray a e, Num e) => (e -> e) -> Matrix a e -> Matrix a e
>mmap f (Matrix x) = Matrix $ array ((li,lj),(ui,uj)) [((i,j),f (x!(i,j))) | i <- range (li,ui), j <- range (lj,uj)] where
>
>	((li,lj),(ui,uj)) = bounds x
>
>mbin :: (IArray a e, Num e) => (e -> e -> e) -> Matrix a e -> Matrix a e -> Matrix a e
>mbin f mx@(Matrix x) my@(Matrix y) 
>	| bounds x == ((0,0),(0,0)) = mmap (\z -> f (x!(0,0)) z) my
>	| bounds y == ((0,0),(0,0)) = mmap (\z -> f z (y!(0,0))) mx
>	| otherwise = Matrix $ matrix_by_element f x y
>
>matrix_by_element :: (IArray a e, Num e) => (e -> e -> e) -> a (Int,Int) e -> a (Int,Int) e -> a (Int,Int) e
>matrix_by_element f x y = array result_bounds [((i,j),f (x!(i,j)) (y!(i,j)))
>	| i <- range (li,ui), j <- range (lj,uj)] where
>
>	((li,lj),(ui,uj)) = bounds x
>	((li',lj'),(ui',uj')) = bounds y
>	result_bounds
>		| ui == ui' && li == li' && uj == uj' && lj == lj' = bounds x
>		| otherwise = error "incompatible matrices"
>
>mmul :: (IArray a e, Num e) => Matrix a e -> Matrix a e -> Matrix a e
>mmul mx@(Matrix x) my@(Matrix y)
>	| bounds x == ((0,0),(0,0)) = mmap (\z -> (x!(0,0)) * z) my
>	| bounds y == ((0,0),(0,0)) = mmap (\z -> z * (y!(0,0))) mx
>	| otherwise = Matrix $ matrix_by_matrix x y
>
>matrix_by_matrix :: (IArray a e, Num e) => a (Int,Int) e -> a (Int,Int) e -> a (Int,Int) e
>matrix_by_matrix x y = array result_bounds [((i,j), sum [x!(i,k) * y!(k,j) | k <- range (lj,uj)])
>	| i <- range (li,ui), j <- range (lj',uj') ] where
>
>	((li,lj),(ui,uj)) = bounds x
>	((li',lj'),(ui',uj')) = bounds y
>	result_bounds
>		| (lj,uj)==(li',ui') = ((li,lj'),(ui,uj'))
>		| otherwise = error "incompatible matrices"
>
>instance (IArray a e, Num e, Eq (Matrix a e), Show (Matrix a e)) => Num (Matrix a e) where
>	fromInteger i = scalar (fromInteger i)
>	abs = mmap abs
>	signum = mmap signum
>	(+) = mbin (+)
>	(-) = mbin (-)
>	(*) = mmul
>
>-- define / properly
>instance (IArray a e, Fractional e, Eq (Matrix a e), Show (Matrix a e)) => Fractional (Matrix a e) where
>	fromRational r = scalar (fromRational r)
>	recip = undefined
>	(/) = undefined
>
>-- define **
>instance (IArray a e, Floating e, Fractional (Matrix a e)) => Floating (Matrix a e) where
>	pi = scalar pi
>	exp = mmap exp
>	log = mmap log
>	sqrt = mmap sqrt
>	logBase = undefined
>	sin = mmap sin
>	cos = mmap cos
>	tan = mmap tan
>	asin = mmap asin
>	acos = mmap acos
>	atan = mmap atan
>	sinh = mmap sinh
>	cosh = mmap cosh
>	tanh = mmap tanh
>	asinh = mmap asinh
>	acosh = mmap acosh
>	atanh = mmap atanh
>
>class (IArray a e, Num e, Eq (Matrix a e), Show (Matrix a e)) => NumMatrix a e where
>	(.*) :: Matrix a e -> Matrix a e -> Matrix a e
>
>instance (IArray a e, Num e, Eq (Matrix a e), Show (Matrix a e)) => NumMatrix a e where
>	(.*) = mbin (*)
>
>class (NumMatrix a e,Fractional e) => FractionalMatrix a e where
>	(./) :: Matrix a e -> Matrix a e -> Matrix a e
>
>instance (NumMatrix a e,Fractional e) => FractionalMatrix a e where
>	(./) = mbin (/)
>
>class (FractionalMatrix a e,Floating e) => FloatingMatrix a e where
>	(.**) :: Matrix a e -> Matrix a e -> Matrix a e
>	expm :: Matrix a e -> Matrix a e -> Matrix a e
>
>instance (FractionalMatrix a e,Floating e) => FloatingMatrix a e where
>	(.**) = mbin (**)
>	expm = undefined

\end{fontsize}
\end{document}
